% ======== runRBFN_IRA_Classification ========
% This script trains an RBF Network on an IRA dataset, and plots the
% resulting score function and decision boundary.
% 
% There are three main steps to the training process:
%   1. Prototype selection through k-means clustering.
%   2. Calculation of beta coefficient (which controls the width of the 
%      RBF neuron activation function) for each RBF neuron.
%   3. Training of output weights for each category using gradient descent.
%   4. Evaluates the RBFN's accuracy on the training set.

% $Author: Jung-shin Lee $    $Date: 2018/08/24 $    $Revision: 1.0 $

% Clear all existing variables from the workspace.
clear;
clc;

% Add the subdirectories to the path.
addpath('kMeans');
addpath('RBFN');

% RBFNN-ELM
%     mode     - 1 : pinv & no regulation term, 2 : inv & including
%                regulation term, 3 : pinv & including regulation term
lambda           = 0.2;
mode             = 3;
featureDim       = 2;
HiddenIter       = 72;
num              = zeros(1, HiddenIter);
TrainingAccuracy = zeros(1, HiddenIter);
TestAcuuracy     = zeros(1, HiddenIter);
TrainingTime     = zeros(1, HiddenIter);
TestTime         = zeros(1, HiddenIter);

% Learning Results
strFileHeader = strcat('DEM_ML_RBF_ELM_Lambda', num2str(lambda), '_RegulationMode', int2str(mode), '.dat'); 
LR  = fopen(strFileHeader, 'w');

for Lnum = 1:1:HiddenIter
    % Load the data set. 
    % This loads two variables, X and y.
    %   X_train - The dataset, 1 sample per row.
    %   Y_train - The corresponding label
    % The data is randomly sorted and grouped by category.
    DBTrainSample = load('TrainDBData3.dat');
    X_train = DBTrainSample(:, 1:2);
    Y_train = DBTrainSample(:, 3);

    inputDim1 = length(X_train(1,:));
    % Set 'm' to the number of data points.
    m1 = size(X_train, 1);

    % ===================================
    %     Train RBF Network
    % ===================================

    disp('Training the RBFN...');

    % Train the RBFN using 'num' centers per category.
    num(1, Lnum) = 70*Lnum;
    H1_num = num(1, Lnum);
    H2_num = floor(H1_num/6);
    H3_num = floor(H2_num/2);
    fprintf('Hidden layer number =  %d\n', num(1, Lnum));
    tic;
    % For Classification
    %[Centers, betas, Theta] = trainRBFN(X_train, Y_train, num(1, Lnum), lambda, mode, true);
    % 1st Hidden layer : AutoEncoder
    [Centers_H1, betas_H1, Theta_H1] = trainFuncApproxRBFN(X_train, Y_train, H1_num, 1, mode, lambda, true);
    New_Y_train = zeros(m1, 1);
    for i = 1:1:m1
        New_Y_train(i,:) = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_train(i, :));
    end
    % New_Y_train = New_Y_train + randn(m1,1)*1.0e-4;
    % 2st Hidden layer
    [Centers_H2, betas_H2, Theta_H2] = trainFuncApproxRBFN(New_Y_train, Y_train, H2_num, 1, mode, lambda, true);
    Train_Time = toc;
    TrainingTime(1, Lnum) = Train_Time;

    % ========================================
    %       Measure Training Accuracy
    % ========================================

    disp('Measuring training accuracy...');
    % For each training sample...
    yfit = zeros(m1, 1);
    for i = 1 : m1
        % Compute the scores for both categories.
        % For classification
        %[scores phis] = evaluateRBFN(Centers, betas, Theta, X_train(i, :));
        % For Regression
        scores1 = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_train(i, :));
        scores = evaluateFuncApproxRBFN(Centers_H2, betas_H2, Theta_H2, 1, scores1);

        yfit(i) = scores;

    end

    LonMean = 127.2604444444444;
    LonStd  = 0.174444444444433;
    LatMean = 35.891611111111111;
    LatStd  = 0.382111111111115;
    HeiMean = 332;
    HeiStd  = 536;
    Lon_train = (X_train(:,1)*LonStd/2.0) + LonMean;
    Lat_train = (X_train(:,2)*LatStd/2.0) + LatMean;
    Hei_train = (Y_train(:,1)*HeiStd/2.0) + HeiMean;
    Hei_trainfit   = (yfit*HeiStd/2.0) + HeiMean;

    ELMErr_train = mean(abs((Hei_trainfit - Hei_train)./Hei_train * 100));
    TrainingAccuracy(1, Lnum) = 100 - ELMErr_train;
    fprintf('Hidden layer = %d, Train Time = %f, Train error = %f%%, Train accuracy: %f%%\n', ...
        num(1, Lnum), TrainingTime(1, Lnum), ELMErr_train, TrainingAccuracy(1, Lnum));
    
    if exist('OCTAVE_VERSION') fflush(stdout); end;

    figure(1)
    plot(Lon_train, Hei_train, 'Linewidth', 1.5); hold on;
    plot(Lon_train, Hei_trainfit, 'Linewidth', 1.5); hold off;
    grid on;
    xlabel('Longitude(Degree)')
    ylabel('Height(Degree)')
    legend('True DEM', 'Trained DEM by ELM')

    figure(2)
    plot(Lat_train, Hei_train, 'Linewidth', 1.5); hold on;
    plot(Lat_train, Hei_trainfit, 'Linewidth', 1.5); hold off;
    grid on;
    xlabel('Latitude(Degree)')
    ylabel('Height(Degree)')
    legend('True DEM', 'Trained DEM by ELM')

    % ========================================
    %       Measure Test Accuracy
    % ========================================
    DBTestSample = load('TestDBData3.dat');
    X_test = DBTestSample(:, 1:2);
    Y_test = DBTestSample(:, 3);

    % Set 'm' to the number of data points.
    m2 = size(X_test, 1);

    disp('Measuring test accuracy...');
    % For each training sample...
    tic;
    yfit_test = zeros(m2, 1);
    for i = 1 : m2
        % Compute the scores for both categories.
        % For Classification
        %[scores phis] = evaluateRBFN(Centers, betas, Theta, X_test(i, :));
        % For Regression
        scores1 = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_test(i, :));
        scores = evaluateFuncApproxRBFN(Centers_H2, betas_H2, Theta_H2, 1, scores1);

        yfit_test(i) = scores;

    end
    Evaluation_time   = toc/m2;
    TestTime(1, Lnum) = Evaluation_time;
    % Mark the incorrectly recognized samples with a black asterisk.
    %plot(wrong(:, 1), wrong(:, 2), 'k*');
    Lon_test = (X_test(:,1)*LonStd/2.0) + LonMean;
    Lat_test = (X_test(:,2)*LatStd/2.0) + LatMean;
    Hei_test = (Y_test(:,1)*HeiStd/2.0) + HeiMean;
    Hei_testfit   = (yfit_test*HeiStd/2.0) + HeiMean;
    ELMErr_test = mean(abs((Hei_testfit - Hei_test)./Hei_test * 100));
    TestAcuuracy(1, Lnum) = 100 - ELMErr_test;
    fprintf('Hidden layer = %d, Test Time = %f, Test error = %f%%, Test accuracy: %f%%\n', ...
        num(1, Lnum), TestTime(1, Lnum), ELMErr_test, TestAcuuracy(1, Lnum));
    
    if exist('OCTAVE_VERSION') fflush(stdout); end
end

DBELMResult = [num; TrainingAccuracy; TestAcuuracy; TrainingTime; TestTime];
strTemp = '';
for idx = 1 : size(DBELMResult, 1)
    strTemp = [strTemp, '%f '];
end
strTemp = [strTemp, '\n '];
fprintf(LR, strTemp, DBELMResult);
fclose(LR);
