% ======== runRBFN_DBMatching_Regression ========
% This script trains an RBF Network on an DEM Level 3 dataset, and plots the
% resulting score function and decision boundary.
% 
% There are three main steps to the training process:
%   1. Prototype selection through k-means clustering.
%   2. Calculation of beta coefficient (which controls the width of the 
%      RBF neuron activation function) for each RBF neuron.
%   3. Training of output weights for each category using gradient descent.
%   4. Evaluates the RBFN's accuracy on the training set.

% $Author: Jung-shin Lee $    $Date: 2018/08/24 $    $Revision: 1.0 $

% Clear all existing variables from the workspace.
% clear;
% clc;

% Add the subdirectories to the path.
addpath('kMeans');
addpath('RBFN');

% RBFNN-ELM
%     mode     - 1 : pinv & no regulation term, 2 : inv & including
%                regulation term, 3 : pinv & including regulation term
lambda           = 0.2;
mode             = 3;
featureDim       = 2;

% Load the data set. 
% This loads two variables, X and y.
%   X - The dataset, 1 sample per row.
%   y - The corresponding label
% The data is randomly sorted and grouped by category.
% DBTrainSample = load('TrainDBData2.dat');
% Wide-Area Training Data
% DBTrainSample = load('Wide_TrainDBData.dat');
DBTrainSample = load('TrainDBData3.dat');

X_train = DBTrainSample(:, 1:2);
Y_train = DBTrainSample(:, 3);

inputDim1 = length(X_train(1,:));
% Set 'm' to the number of data points.
m1 = size(X_train, 1);

% ===================================
%     Train RBF Network
% ===================================

disp('Training the RBFN...');

% Train the RBFN using 'num' centers per category.
H1_num = 4200;
H2_num = floor(H1_num/6);
H3_num = floor(H2_num/2);
fprintf('Hidden layer number =  %d, %d, %d \n', H1_num, H2_num, H3_num);
tic;
% 1st Hidden layer : AutoEncoder
[Centers_H1, betas_H1, Theta_H1] = trainFuncApproxRBFN(X_train, Y_train, H1_num, 1, mode, lambda, true);
New_Y_train = zeros(m1, 1);
for i = 1:1:m1
    New_Y_train(i,:) = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_train(i, :));
end
% New_Y_train = New_Y_train + randn(m1,1)*1.0e-4;
% 2st Hidden layer
[Centers_H2, betas_H2, Theta_H2] = trainFuncApproxRBFN(New_Y_train, Y_train, H2_num, 1, mode, lambda, true);
% New_Y_train2 = zeros(m1, 1);
% for i = 1:1:m1
%     New_Y_train2(i,:) = evaluateFuncApproxRBFN(Centers_H2, betas_H2, Theta_H2, 1, New_Y_train(i));
% end
% New_Y_train2 = New_Y_train2 + randn(m1,1)*1.0e-4;
% % 3st Hidden layer
% [Centers_H3, betas_H3, Theta_H3] = trainFuncApproxRBFN(New_Y_train2, Y_train, H3_num, 1, mode, lambda, true);

Train_Time = toc;
TrainingTime = Train_Time;

save('TrainingParameter_ML_RBF_ELM_Wide3.mat', 'Centers_H1', 'betas_H1', 'Theta_H1', 'Centers_H2', 'betas_H2', 'Theta_H2'); % 'Centers_H3', 'betas_H3', 'Theta_H3');

% ========================================
%       Measure Training Accuracy
% ========================================

disp('Measuring training accuracy...');

% For each training sample...
yfit = zeros(m1, 1);
for i = 1 : m1
    % Compute the scores for both categories.
        % For classification
        %[scores phis] = evaluateRBFN(Centers, betas, Theta, X_train(i, :));
        % For Regression
        scores1 = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_train(i, :));
        scores = evaluateFuncApproxRBFN(Centers_H2, betas_H2, Theta_H2, 1, scores1);
%         scores = evaluateFuncApproxRBFN(Centers_H3, betas_H3, Theta_H3, 1, scores2);

        yfit(i) = scores;

end

% Mark the incorrectly recognized samples with a black asterisk.
% LonMean = 127.2604444444444;
% LonStd  = 0.174222222222213;
% LatMean = 35.891611111111111;
% LatStd  = 0.382111111111115;
% HeiMean = 332;
% HeiStd  = 536;
% Wide-Area Training Data
% LonMean = 127.2610000000000;
% LonStd  = 0.176444444444442;
% LatMean = 35.892944444444446;
% LatStd  = 0.384333333333331;
% HeiMean = 332;
% HeiStd  = 536;
LonMean = 127.2604444444444;
LonStd  = 0.174444444444433;
LatMean = 35.891611111111111;
LatStd  = 0.382111111111115;
HeiMean = 332;
HeiStd  = 536;
Lon_train = (X_train(:,1)*LonStd/2.0) + LonMean;
Lat_train = (X_train(:,2)*LatStd/2.0) + LatMean;
Hei_train = (Y_train(:,1)*HeiStd/2.0) + HeiMean;
Hei_trainfit   = (yfit*HeiStd/2.0) + HeiMean;

ELMErr_train = mean(abs((Hei_trainfit - Hei_train)./Hei_train * 100));
TrainingAccuracy = 100 - ELMErr_train;
fprintf('Hidden layer = %d, %d, %d, Train Time = %f, Train error = %f%%, Train accuracy: %f%%\n', ...
    H1_num, H2_num, H3_num, TrainingTime, ELMErr_train, TrainingAccuracy);

if exist('OCTAVE_VERSION') fflush(stdout); end

figure(1)
% plot(Lon_train, Hei_train, 'Linewidth', 1.5); hold on;
plot(Lon_train, Hei_train, '.', 'Markersize', 6); hold on;
% plot(Lon_train, Hei_trainfit, 'Linewidth', 1.5); hold off;
plot(Lon_train, Hei_trainfit, '.', 'Markersize', 6); hold off;
grid on;
xlabel('Longitude(Degree)')
ylabel('Height(Degree)')
legend('True DEM', 'Trained DEM by ML-RBF-ELM')

figure(2)
plot(Lat_train, Hei_train, 'Linewidth', 1.5); hold on;
plot(Lat_train, Hei_trainfit, 'Linewidth', 1.5); hold off;
grid on;
xlabel('Latitude(Degree)')
ylabel('Height(Degree)')
legend('True DEM', 'Trained DEM by ML-RBF-ELM')

% ========================================
%       Measure Test Accuracy
% ========================================
% DBTestSample = load('TestDBData2.dat');
% Wide-area Test Data
% DBTestSample = load('Wide_TestDBData.dat');
DBTestSample = load('TestDBData3.dat');

X_test = DBTestSample(:, 1:2);
Y_test = DBTestSample(:, 3);

% Set 'm' to the number of data points.
m2 = size(X_test, 1);

disp('Measuring test accuracy...');
% For each training sample...
tic;
yfit_test = zeros(m2, 1);
for i = 1 : m2
    % Compute the scores for both categories.
    % For Classification
    %[scores phis] = evaluateRBFN(Centers, betas, Theta, X_test(i, :));
    % For Regression
    scores1 = evaluateFuncApproxRBFN(Centers_H1, betas_H1, Theta_H1, 1, X_test(i, :));
    scores = evaluateFuncApproxRBFN(Centers_H2, betas_H2, Theta_H2, 1, scores1);
%     scores = evaluateFuncApproxRBFN(Centers_H3, betas_H3, Theta_H3, 1, scores2);

    yfit_test(i) = scores;

end
Evaluation_time   = toc/m2;
TestTime = Evaluation_time;
% Mark the incorrectly recognized samples with a black asterisk.
%plot(wrong(:, 1), wrong(:, 2), 'k*');
Lon_test = (X_test(:,1)*LonStd/2.0) + LonMean;
Lat_test = (X_test(:,2)*LatStd/2.0) + LatMean;
Hei_test = (Y_test(:,1)*HeiStd/2.0) + HeiMean;
Hei_testfit   = (yfit_test*HeiStd/2.0) + HeiMean;
ELMErr_test = mean(abs((Hei_testfit - Hei_test)./Hei_test * 100));
TestAcuuracy = 100 - ELMErr_test;
fprintf('Hidden layer = %d, %d, %d, Test Time = %f, Test error = %f%%, Test accuracy: %f%%\n', ...
    H1_num, H2_num, H3_num, TestTime, ELMErr_test, TestAcuuracy);
if exist('OCTAVE_VERSION') fflush(stdout); end

% FileOut3 = fopen('ML_RBF_ELMTrainResult.dat', 'w');
% CIWELM_TrainResult = [Lon_train Lat_train Hei_trainfit];
% tempMat = CIWELM_TrainResult';
% strTemp = '';
% for idx = 1 : size(tempMat, 1)
%     strTemp = [strTemp, '%f '];
% end
% strTemp = [strTemp, '\n '];
% fprintf(FileOut3, strTemp, tempMat);
% fclose(FileOut3);
% 
% FileOut4 = fopen('ML_RBF_ELMTestResult.dat', 'w');
% CIWELM_TestResult = [Lon_test Lat_test Hei_testfit];
% tempMat = CIWELM_TestResult';
% strTemp = '';
% for idx = 1 : size(tempMat, 1)
%     strTemp = [strTemp, '%f '];
% end
% strTemp = [strTemp, '\n '];
% fprintf(FileOut4, strTemp, tempMat);
% fclose(FileOut4);